// Copyright (c) 2010, Aldis Berjoza <aldis@bsdroot.lv>
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following disclaimer
//    in the documentation and/or other materials provided with the
//    distribution.
// 3. Neither the name of the  nor the names of its
//    contributors may be used to endorse or promote products derived from
//    this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


// vim:tabstop=4:shiftwidth=4
// REFERENCE:	http://www.purplepixie.org/cgi/howto.php
// 				http://www.ietf.org/rfc/rfc2965.txt
// 				http://msdn.microsoft.com/en-us/library/aa384321(VS.85).aspx

//Set-Cookie: name=value; expires=date; path=path; domain=domain; secure

#include <assert.h>

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cgi.h"



struct s_KeyValue *cgiGet(void) {	// {{{1
	char	*qget_ptr = NULL;

	if ((qget_ptr = getenv("QUERY_STRING")) == NULL) return NULL;
	assert(qget_ptr != NULL);

	int l = strlen(qget_ptr);
	if (! l) return NULL;

	if (l > CGI_MAX_GET_LEN) {
		fprintf(stderr, "GET too long. Shrinking\n");
		l = CGI_MAX_GET_LEN;
	}

	struct s_KeyValue *cgi_get = malloc(sizeof(struct s_KeyValue *));
	assert(cgi_get != NULL);

	cgi_get->len = l;

	cgi_get->data = malloc(cgi_get->len + 1);
	assert(cgi_get->data != NULL);

	int j = 0;

	for (int i = 0; i < cgi_get->len; i++) {
		if (qget_ptr[i] == '&') {
			cgi_get->data[i-j] = 0;
		}
		else if (qget_ptr[i] == '%') {
			cgi_get->data[i-j] = cgiURLDecodeChar(qget_ptr + i);
			cgi_get->data[i+1] = 0;
			cgi_get->data[i+2] = 0;
			i += 2;
			j += 2;
		}
		else {
			if (j) {
				cgi_get->data[i-j] = qget_ptr[i];
				cgi_get->data[i] = 0;
			}
			else {
				cgi_get->data[i] = qget_ptr[i];
			}
		}
	}

	cgi_get->data[l] = 0;
	cgi_get->len -= j;
	return cgi_get;
}
// 1}}}

struct s_KeyValue *cgiPost(void) {	// {{{1
	char *tlen_ptr = NULL;

	if ((tlen_ptr = getenv("CONTENT_LENGTH")) == NULL) return NULL;
	assert(tlen_ptr != NULL);

	int l = atoi(tlen_ptr);

	if (! l) return NULL;

	if (l > CGI_MAX_POST_LEN) {
		fprintf(stderr, "POST too long. Shrinking\n");
		l = CGI_MAX_POST_LEN;
	}

	struct s_KeyValue *cgi_post = malloc(sizeof(struct s_KeyValue *));
	assert(cgi_post != NULL);

	cgi_post->len = l;

	cgi_post->data = malloc(cgi_post->len + 1);	// WTF is this changing cgi_get->len
	assert(cgi_post->data != NULL);


//	fgets(cgi_post->data, l, stdin);

	int j = 0;
/*
	for (int i = 0; i < l; i++) {
		if (cgi_post->data[i] == '&') {
			cgi_post->data[i-j] = 0;
		}
		else if (cgi_post->data[i] == '%') {
			cgi_post->data[i-j] = cgiURLDecodeChar(cgi_post->data + i);
			cgi_post->data[i+1] = 0;
			cgi_post->data[i+2] = 0;
			i += 2;
			j += 2;
		}
		else {
			if (j) {
				cgi_post->data[i-j] = cgi_post->data[i];
				cgi_post->data[i] = 0;
			}
			else {
				cgi_post->data[i] = cgi_post->data[i];
			}
		}
	}
*/
	cgi_post->data[l] = 0;
	cgi_post->len -= j;
	return cgi_post;
}
// 1}}}

char cgiURLDecodeChar(const char *encodedChr) {	// {{{1
	assert(encodedChr != NULL);

	if (encodedChr[0] == '%') {
		if (isxdigit(encodedChr[1]) && isxdigit(encodedChr[2])) {
			int a = encodedChr[1] - 48;
			int b = encodedChr[2] - 48;
			if (a > 9) a -= 7;
			if (b > 9) b -= 7;
			return (char)(a * 16 + b);
		}
	}

	return encodedChr[0];
}
// 1}}}

// vim:tabstop=4:shiftwidth=4
