// Copyright (c) 2010, Aldis Berjoza <aldis@bsdroot.lv>
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above
//    copyright notice, this list of conditions and the following disclaimer
//    in the documentation and/or other materials provided with the
//    distribution.
// 3. Neither the name of the  nor the names of its
//    contributors may be used to endorse or promote products derived from
//    this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <assert.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "config.h"

#define CONF_LOADABLE_FILE_MAX_SIZE		10240	// 10KB should be nough for config file;


char *Value(const char *restrict key, const struct s_KeyValue *restrict kv_ptr) {	// {{{1
	if (key == NULL) {
		fprintf(stderr, "Invalid pointer to key\n");
		assert(0);
		exit(1);
	}

	if (strlen(key) == 0) {
		fprintf(stderr, "Invalid option name\n");
		assert(0);
		exit(1);
	}

	if (kv_ptr == NULL) return NULL;

	int l = strlen(key);

	for (int i = 0; i < kv_ptr->len; i++) {
		if (key[0] == (kv_ptr->data)[i]) {
			int found = 1;

			for (int j = 1; j < l; j++) {
				if (key[j] != (kv_ptr->data)[i+j]) {
					found = 0;
					break;
				}
			}

			if (found == 1 && (kv_ptr->data)[i+l] == '=') return kv_ptr->data + i + l + 1;
		}
	}

#ifdef LOCAL_DEBUG
	fprintf(stderr, "Option '%s' not found\n", key);
#endif
	return NULL;
}
// 1}}}

void KeyValueFree(struct s_KeyValue *kv_ptr) {	// {{{1
	if (kv_ptr != NULL) {
		free(kv_ptr->data);
		free(kv_ptr);
	}
}
// 1}}}

// vim:tabstop=4:shiftwidth=4
